# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Snuglite(KaitaiStruct):
    """:field destination_callsign: beacon_types.type_check.destination_callsign
    :field source_callsign: beacon_types.type_check.source_callsign
    :field csp_header_priority: beacon_types.type_check.csp_header_priority
    :field csp_header_source: beacon_types.type_check.csp_header_source
    :field csp_header_destination: beacon_types.type_check.csp_header_destination
    :field csp_header_destination_port: beacon_types.type_check.csp_header_destination_port
    :field csp_header_source_port: beacon_types.type_check.csp_header_source_port
    :field csp_header_reserved: beacon_types.type_check.csp_header_reserved
    :field csp_header_flags: beacon_types.type_check.csp_header_flags
    :field firmware_version: beacon_types.type_check.firmware_version
    :field positioning_flag: beacon_types.type_check.positioning_flag
    :field position_x: beacon_types.type_check.position_x
    :field position_y: beacon_types.type_check.position_y
    :field position_z: beacon_types.type_check.position_z
    :field velocity_x: beacon_types.type_check.velocity_x
    :field velocity_y: beacon_types.type_check.velocity_y
    :field velocity_z: beacon_types.type_check.velocity_z
    :field battery_mode: beacon_types.type_check.battery_mode
    :field battery_voltage: beacon_types.type_check.battery_voltage
    :field battery_current: beacon_types.type_check.battery_current
    :field power_switch_gps_side: beacon_types.type_check.power_switch_gps_side
    :field power_switch_magnetometer: beacon_types.type_check.power_switch_magnetometer
    :field power_switch_sd_card: beacon_types.type_check.power_switch_sd_card
    :field power_switch_gps_up: beacon_types.type_check.power_switch_gps_up
    :field power_switch_uhf: beacon_types.type_check.power_switch_uhf
    :field power_switch_boom: beacon_types.type_check.power_switch_boom
    :field power_supply_current_boom: beacon_types.type_check.power_supply_current_boom
    :field power_supply_current_uhf: beacon_types.type_check.power_supply_current_uhf
    :field power_supply_current_gps_up: beacon_types.type_check.power_supply_current_gps_up
    :field power_supply_current_sd_card: beacon_types.type_check.power_supply_current_sd_card
    :field power_supply_current_magnetometer: beacon_types.type_check.power_supply_current_magnetometer
    :field power_supply_current_gps_side: beacon_types.type_check.power_supply_current_gps_side
    :field solar_cell_input_voltage_x: beacon_types.type_check.solar_cell_input_voltage_x
    :field solar_cell_input_voltage_y: beacon_types.type_check.solar_cell_input_voltage_y
    :field solar_cell_input_voltage_z: beacon_types.type_check.solar_cell_input_voltage_z
    :field solar_cell_input_current_x: beacon_types.type_check.solar_cell_input_current_x
    :field solar_cell_input_current_y: beacon_types.type_check.solar_cell_input_current_y
    :field solar_cell_input_current_z: beacon_types.type_check.solar_cell_input_current_z
    :field estimated_attitude_q0: beacon_types.type_check.estimated_attitude_q0
    :field estimated_attitude_q1: beacon_types.type_check.estimated_attitude_q1
    :field estimated_attitude_q2: beacon_types.type_check.estimated_attitude_q2
    :field estimated_attitude_q3: beacon_types.type_check.estimated_attitude_q3
    :field estimated_gyro_bias_roll: beacon_types.type_check.estimated_gyro_bias_roll
    :field estimated_gyro_bias_pitch: beacon_types.type_check.estimated_gyro_bias_pitch
    :field estimated_gyro_bias_yaw: beacon_types.type_check.estimated_gyro_bias_yaw
    :field estimated_angular_rate_roll: beacon_types.type_check.estimated_angular_rate_roll
    :field estimated_angular_rate_pitch: beacon_types.type_check.estimated_angular_rate_pitch
    :field estimated_angular_rate_yaw: beacon_types.type_check.estimated_angular_rate_yaw
    :field measured_angular_rate_roll: beacon_types.type_check.measured_angular_rate_roll
    :field measured_angular_rate_pitch: beacon_types.type_check.measured_angular_rate_pitch
    :field measured_angular_rate_yaw: beacon_types.type_check.measured_angular_rate_yaw
    :field sun_eclipse: beacon_types.type_check.sun_eclipse
    :field attitude_convergence: beacon_types.type_check.attitude_convergence
    :field attitude_variance_q0: beacon_types.type_check.attitude_variance_q0
    :field attitude_variance_q1: beacon_types.type_check.attitude_variance_q1
    :field attitude_variance_q2: beacon_types.type_check.attitude_variance_q2
    :field attitude_variance_q3: beacon_types.type_check.attitude_variance_q3
    :field current_operation_mode: beacon_types.type_check.current_operation_mode
    :field elapsed_time: beacon_types.type_check.elapsed_time
    :field temperature_solar_panel_plus_x: beacon_types.type_check.temperature_solar_panel_plus_x
    :field temperature_solar_panel_plus_y: beacon_types.type_check.temperature_solar_panel_plus_y
    :field temperature_solar_panel_minus_x: beacon_types.type_check.temperature_solar_panel_minus_x
    :field temperature_solar_panel_minus_y: beacon_types.type_check.temperature_solar_panel_minus_y
    :field temperature_solar_panel_minus_z: beacon_types.type_check.temperature_solar_panel_minus_z
    :field temperature_obc_1: beacon_types.type_check.temperature_obc_1
    :field temperature_obc_2: beacon_types.type_check.temperature_obc_2
    :field temperature_eps_module_1: beacon_types.type_check.temperature_eps_module_1
    :field temperature_eps_module_2: beacon_types.type_check.temperature_eps_module_2
    :field temperature_eps_module_3: beacon_types.type_check.temperature_eps_module_3
    :field temperature_eps_module_4: beacon_types.type_check.temperature_eps_module_4
    :field temperature_uhf_module_1: beacon_types.type_check.temperature_uhf_module_1
    :field temperature_uhf_module_2: beacon_types.type_check.temperature_uhf_module_2
    :field boom_release_status: beacon_types.type_check.boom_release_status
    :field antenna_release_status: beacon_types.type_check.antenna_release_status
    :field count_antenna_release_trial: beacon_types.type_check.count_antenna_release_trial
    :field count_boom_release_trial: beacon_types.type_check.count_boom_release_trial
    :field beacon_type: beacon_types.type_check.beacon_type
    :field satellite_time: beacon_types.type_check.satellite_time
    
    .. seealso::
       Source - https://snuglitecubesat.wixsite.com/website/post/snuglite-beacon-structure
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.beacon_types = Snuglite.BeaconTypesT(self._io, self, self._root)

    class BeaconTypesT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 1011438661:
                self.type_check = Snuglite.Simple(self._io, self, self._root)
            else:
                self.type_check = Snuglite.Full(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            _pos = self._io.pos()
            self._io.seek(61)
            self._m_check = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_check', None)


    class Full(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.destination_callsign = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not self.destination_callsign == u"DS0DH":
                raise kaitaistruct.ValidationNotEqualError(u"DS0DH", self.destination_callsign, self._io, u"/types/full/seq/0")
            self.last_digit_of_destination_callsign_and_destination_ssid = self._io.read_u2be()
            self.source_callsign = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not self.source_callsign == u"DS0DH":
                raise kaitaistruct.ValidationNotEqualError(u"DS0DH", self.source_callsign, self._io, u"/types/full/seq/2")
            self.last_digit_of_source_callsign_and_source_ssid = self._io.read_u2be()
            self.control_and_pid = self._io.read_u2be()
            self.csp_header_priority = self._io.read_bits_int_be(2)
            self.csp_header_source = self._io.read_bits_int_be(5)
            self.csp_header_destination = self._io.read_bits_int_be(5)
            self.csp_header_destination_port = self._io.read_bits_int_be(6)
            self.csp_header_source_port = self._io.read_bits_int_be(6)
            self.csp_header_reserved = self._io.read_bits_int_be(4)
            self.csp_header_flags = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.start_id = (self._io.read_bytes(6)).decode(u"UTF-8")
            if not self.start_id == u"SNUGL>":
                raise kaitaistruct.ValidationNotEqualError(u"SNUGL>", self.start_id, self._io, u"/types/full/seq/12")
            self.firmware_version = self._io.read_u1()
            self.time_year = self._io.read_u1()
            self.time_month = self._io.read_u1()
            self.time_day = self._io.read_u1()
            self.time_hour = self._io.read_u1()
            self.time_minute = self._io.read_u1()
            self.time_second = self._io.read_u1()
            self.positioning_flag = self._io.read_u1()
            self.position_x = self._io.read_s4be()
            self.position_y = self._io.read_s4be()
            self.position_z = self._io.read_s4be()
            self.velocity_x = self._io.read_s4be()
            self.velocity_y = self._io.read_s4be()
            self.velocity_z = self._io.read_s4be()
            self.battery_mode = self._io.read_u1()
            self.battery_voltage = self._io.read_u2be()
            self.battery_current = self._io.read_u2be()
            self.not_used = self._io.read_bits_int_be(2)
            self.power_switch_gps_side = self._io.read_bits_int_be(1) != 0
            self.power_switch_magnetometer = self._io.read_bits_int_be(1) != 0
            self.power_switch_sd_card = self._io.read_bits_int_be(1) != 0
            self.power_switch_gps_up = self._io.read_bits_int_be(1) != 0
            self.power_switch_uhf = self._io.read_bits_int_be(1) != 0
            self.power_switch_boom = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.power_supply_current_boom = self._io.read_u2be()
            self.power_supply_current_uhf = self._io.read_u2be()
            self.power_supply_current_gps_up = self._io.read_u2be()
            self.power_supply_current_sd_card = self._io.read_u2be()
            self.power_supply_current_magnetometer = self._io.read_u2be()
            self.power_supply_current_gps_side = self._io.read_u2be()
            self.solar_cell_input_voltage_x = self._io.read_u2be()
            self.solar_cell_input_voltage_y = self._io.read_u2be()
            self.solar_cell_input_voltage_z = self._io.read_u2be()
            self.solar_cell_input_current_x = self._io.read_u2be()
            self.solar_cell_input_current_y = self._io.read_u2be()
            self.solar_cell_input_current_z = self._io.read_u2be()
            self.estimated_attitude_q0 = self._io.read_f4be()
            self.estimated_attitude_q1 = self._io.read_f4be()
            self.estimated_attitude_q2 = self._io.read_f4be()
            self.estimated_attitude_q3 = self._io.read_f4be()
            self.estimated_gyro_bias_roll = self._io.read_f4be()
            self.estimated_gyro_bias_pitch = self._io.read_f4be()
            self.estimated_gyro_bias_yaw = self._io.read_f4be()
            self.estimated_angular_rate_roll = self._io.read_f4be()
            self.estimated_angular_rate_pitch = self._io.read_f4be()
            self.estimated_angular_rate_yaw = self._io.read_f4be()
            self.measured_angular_rate_roll = self._io.read_f4be()
            self.measured_angular_rate_pitch = self._io.read_f4be()
            self.measured_angular_rate_yaw = self._io.read_f4be()
            self.not_used_1 = self._io.read_bits_int_be(6)
            self.sun_eclipse = self._io.read_bits_int_be(1) != 0
            self.attitude_convergence = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.attitude_variance_q0 = self._io.read_f4be()
            self.attitude_variance_q1 = self._io.read_f4be()
            self.attitude_variance_q2 = self._io.read_f4be()
            self.attitude_variance_q3 = self._io.read_f4be()
            self.current_operation_mode = self._io.read_u1()
            self.elapsed_time = self._io.read_s4be()
            self.temperature_solar_panel_plus_x = self._io.read_s1()
            self.temperature_solar_panel_plus_y = self._io.read_s1()
            self.temperature_solar_panel_minus_x = self._io.read_s1()
            self.temperature_solar_panel_minus_y = self._io.read_s1()
            self.temperature_solar_panel_minus_z = self._io.read_s1()
            self.temperature_obc_1 = self._io.read_s1()
            self.temperature_obc_2 = self._io.read_s1()
            self.temperature_eps_module_1 = self._io.read_s1()
            self.temperature_eps_module_2 = self._io.read_s1()
            self.temperature_eps_module_3 = self._io.read_s1()
            self.temperature_eps_module_4 = self._io.read_s1()
            self.temperature_uhf_module_1 = self._io.read_s1()
            self.temperature_uhf_module_2 = self._io.read_s1()
            self.not_used_2 = self._io.read_bits_int_be(6)
            self.boom_release_status = self._io.read_bits_int_be(1) != 0
            self.antenna_release_status = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.count_antenna_release_trial = self._io.read_u1()
            self.count_boom_release_trial = self._io.read_u1()
            self.end_id = (self._io.read_bytes(4)).decode(u"UTF-8")
            if not self.end_id == u"<ITE":
                raise kaitaistruct.ValidationNotEqualError(u"<ITE", self.end_id, self._io, u"/types/full/seq/89")

        @property
        def month(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_month'):
                return self._m_month

            self._m_month = (u"0" + str(self.time_month) if len(str(self.time_month)) == 1 else str(self.time_month))
            return getattr(self, '_m_month', None)

        @property
        def minute(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_minute'):
                return self._m_minute

            self._m_minute = (u"0" + str(self.time_minute) if len(str(self.time_minute)) == 1 else str(self.time_minute))
            return getattr(self, '_m_minute', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            self._m_satellite_time = u"20" + self.year + u"-" + self.month + u"-" + self.day + u"T" + self.hour + u":" + self.minute + u":" + self.second + u"Z"
            return getattr(self, '_m_satellite_time', None)

        @property
        def year(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_year'):
                return self._m_year

            self._m_year = (u"0" + str(self.time_year) if len(str(self.time_year)) == 1 else str(self.time_year))
            return getattr(self, '_m_year', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = u"full"
            return getattr(self, '_m_beacon_type', None)

        @property
        def second(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_second'):
                return self._m_second

            self._m_second = (u"0" + str(self.time_second) if len(str(self.time_second)) == 1 else str(self.time_second))
            return getattr(self, '_m_second', None)

        @property
        def day(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_day'):
                return self._m_day

            self._m_day = (u"0" + str(self.time_day) if len(str(self.time_day)) == 1 else str(self.time_day))
            return getattr(self, '_m_day', None)

        @property
        def hour(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_hour'):
                return self._m_hour

            self._m_hour = (u"0" + str(self.time_hour) if len(str(self.time_hour)) == 1 else str(self.time_hour))
            return getattr(self, '_m_hour', None)


    class Simple(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.destination_callsign = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not self.destination_callsign == u"DS0DH":
                raise kaitaistruct.ValidationNotEqualError(u"DS0DH", self.destination_callsign, self._io, u"/types/simple/seq/0")
            self.last_digit_of_destination_callsign_and_destination_ssid = self._io.read_u2be()
            self.source_callsign = (self._io.read_bytes(5)).decode(u"UTF-8")
            if not self.source_callsign == u"DS0DH":
                raise kaitaistruct.ValidationNotEqualError(u"DS0DH", self.source_callsign, self._io, u"/types/simple/seq/2")
            self.last_digit_of_source_callsign_and_source_ssid = self._io.read_u2be()
            self.control_and_pid = self._io.read_u2be()
            self.csp_header_priority = self._io.read_bits_int_be(2)
            self.csp_header_source = self._io.read_bits_int_be(5)
            self.csp_header_destination = self._io.read_bits_int_be(5)
            self.csp_header_destination_port = self._io.read_bits_int_be(6)
            self.csp_header_source_port = self._io.read_bits_int_be(6)
            self.csp_header_reserved = self._io.read_bits_int_be(4)
            self.csp_header_flags = self._io.read_bits_int_be(4)
            self._io.align_to_byte()
            self.start_id = (self._io.read_bytes(6)).decode(u"UTF-8")
            if not self.start_id == u"SNUGL>":
                raise kaitaistruct.ValidationNotEqualError(u"SNUGL>", self.start_id, self._io, u"/types/simple/seq/12")
            self.firmware_version = self._io.read_u1()
            self.time_year = self._io.read_u1()
            self.time_month = self._io.read_u1()
            self.time_day = self._io.read_u1()
            self.time_hour = self._io.read_u1()
            self.time_minute = self._io.read_u1()
            self.time_second = self._io.read_u1()
            self.positioning_flag = self._io.read_u1()
            self.position_x = self._io.read_s4be()
            self.position_y = self._io.read_s4be()
            self.position_z = self._io.read_s4be()
            self.velocity_x = self._io.read_s4be()
            self.velocity_y = self._io.read_s4be()
            self.velocity_z = self._io.read_s4be()
            self.battery_mode = self._io.read_u1()
            self.battery_voltage = self._io.read_u2be()
            self.end_id = (self._io.read_bytes(4)).decode(u"UTF-8")
            if not self.end_id == u"<ITE":
                raise kaitaistruct.ValidationNotEqualError(u"<ITE", self.end_id, self._io, u"/types/simple/seq/29")

        @property
        def month(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_month'):
                return self._m_month

            self._m_month = (u"0" + str(self.time_month) if len(str(self.time_month)) == 1 else str(self.time_month))
            return getattr(self, '_m_month', None)

        @property
        def minute(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_minute'):
                return self._m_minute

            self._m_minute = (u"0" + str(self.time_minute) if len(str(self.time_minute)) == 1 else str(self.time_minute))
            return getattr(self, '_m_minute', None)

        @property
        def satellite_time(self):
            if hasattr(self, '_m_satellite_time'):
                return self._m_satellite_time

            self._m_satellite_time = u"20" + self.year + u"-" + self.month + u"-" + self.day + u"T" + self.hour + u":" + self.minute + u":" + self.second + u"Z"
            return getattr(self, '_m_satellite_time', None)

        @property
        def year(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_year'):
                return self._m_year

            self._m_year = (u"0" + str(self.time_year) if len(str(self.time_year)) == 1 else str(self.time_year))
            return getattr(self, '_m_year', None)

        @property
        def beacon_type(self):
            if hasattr(self, '_m_beacon_type'):
                return self._m_beacon_type

            self._m_beacon_type = u"simple"
            return getattr(self, '_m_beacon_type', None)

        @property
        def second(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_second'):
                return self._m_second

            self._m_second = (u"0" + str(self.time_second) if len(str(self.time_second)) == 1 else str(self.time_second))
            return getattr(self, '_m_second', None)

        @property
        def day(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_day'):
                return self._m_day

            self._m_day = (u"0" + str(self.time_day) if len(str(self.time_day)) == 1 else str(self.time_day))
            return getattr(self, '_m_day', None)

        @property
        def hour(self):
            """only for calculation, do not display."""
            if hasattr(self, '_m_hour'):
                return self._m_hour

            self._m_hour = (u"0" + str(self.time_hour) if len(str(self.time_hour)) == 1 else str(self.time_hour))
            return getattr(self, '_m_hour', None)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Real(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field ccsds_header_first: ax25_frame.ccsds_header.ccsds_header_first
    :field length: ax25_frame.ccsds_header.length
    :field ccsds_header_second: ax25_frame.ccsds_header.ccsds_header_second
    :field service_type: ax25_frame.real_header.service_type
    :field service_sub_type: ax25_frame.real_header.service_sub_type
    :field structure_id: ax25_frame.real_header.structure_id
    :field syncword: ax25_frame.real_header.syncword.syncword
    :field bat_charging_status: ax25_frame.payload.bat_charging_status
    :field v5_bat_current: ax25_frame.payload.v5_bat_current
    :field v3_3_bat_current: ax25_frame.payload.v3_3_bat_current
    :field vbat_bat_current: ax25_frame.payload.vbat_bat_current
    :field v3_3_bat_voltage: ax25_frame.payload.v3_3_bat_voltage
    :field v5_bat_voltage: ax25_frame.payload.v5_bat_voltage
    :field vbat_bat_voltage: ax25_frame.payload.vbat_bat_voltage
    :field bat_board_temperature: ax25_frame.payload.bat_board_temperature
    :field bat_cell_1_temperature: ax25_frame.payload.bat_cell_1_temperature
    :field bat_cell_2_temperature: ax25_frame.payload.bat_cell_2_temperature
    :field bat_cell_3_temperature: ax25_frame.payload.bat_cell_3_temperature
    :field bat_cell_4_temperature: ax25_frame.payload.bat_cell_4_temperature
    :field battery_heater_status_0: ax25_frame.payload.battery_heater_status_0
    :field battery_heater_status_1: ax25_frame.payload.battery_heater_status_1
    :field battery_heater_status_2: ax25_frame.payload.battery_heater_status_2
    :field battery_heater_status_3: ax25_frame.payload.battery_heater_status_3
    :field vbat_eps_voltage: ax25_frame.payload.vbat_eps_voltage
    :field v3_3_eps_voltage: ax25_frame.payload.v3_3_eps_voltage
    :field v5_eps_voltage: ax25_frame.payload.v5_eps_voltage
    :field v12_eps_voltage: ax25_frame.payload.v12_eps_voltage
    :field vbat_eps_current: ax25_frame.payload.vbat_eps_current
    :field v3_3_eps_current: ax25_frame.payload.v3_3_eps_current
    :field v5_eps_current: ax25_frame.payload.v5_eps_current
    :field v12_eps_current: ax25_frame.payload.v12_eps_current
    :field sw1_unused_voltage: ax25_frame.payload.sw1_unused_voltage
    :field sw2_unused_voltage: ax25_frame.payload.sw2_unused_voltage
    :field sw3_instrument_voltage: ax25_frame.payload.sw3_instrument_voltage
    :field sw4_li2_voltage: ax25_frame.payload.sw4_li2_voltage
    :field sw5_uhf_temp_sensor_voltage: ax25_frame.payload.sw5_uhf_temp_sensor_voltage
    :field sw6_unused_voltage: ax25_frame.payload.sw6_unused_voltage
    :field sw7_unused_voltage: ax25_frame.payload.sw7_unused_voltage
    :field sw8_xact_serial_voltage: ax25_frame.payload.sw8_xact_serial_voltage
    :field sw9_gps_voltage: ax25_frame.payload.sw9_gps_voltage
    :field sw10_instrument_lvds_voltage: ax25_frame.payload.sw10_instrument_lvds_voltage
    :field sw1_unused_current: ax25_frame.payload.sw1_unused_current
    :field sw2_unused_current: ax25_frame.payload.sw2_unused_current
    :field sw3_instrument_current: ax25_frame.payload.sw3_instrument_current
    :field sw4_li2_current: ax25_frame.payload.sw4_li2_current
    :field sw5_uhf_temp_sensor_current: ax25_frame.payload.sw5_uhf_temp_sensor_current
    :field sw6_unused_current: ax25_frame.payload.sw6_unused_current
    :field sw7_unused_current: ax25_frame.payload.sw7_unused_current
    :field sw8_xact_serial_current: ax25_frame.payload.sw8_xact_serial_current
    :field sw9_gps_current: ax25_frame.payload.sw9_gps_current
    :field sw10_instrument_lvds_current: ax25_frame.payload.sw10_instrument_lvds_current
    :field eps_mb_temperature: ax25_frame.payload.eps_mb_temperature
    :field eps_db_temperature: ax25_frame.payload.eps_db_temperature
    :field sa1_salw_inner_voltage: ax25_frame.payload.sa1_salw_inner_voltage
    :field sa2_salw_outer_voltage: ax25_frame.payload.sa2_salw_outer_voltage
    :field sa4_sarw_inner_voltage: ax25_frame.payload.sa4_sarw_inner_voltage
    :field sa5_sarw_outer_voltage: ax25_frame.payload.sa5_sarw_outer_voltage
    :field sa1_salw_inner_current: ax25_frame.payload.sa1_salw_inner_current
    :field sa2_salw_outer_current: ax25_frame.payload.sa2_salw_outer_current
    :field sa4_sarw_inner_current: ax25_frame.payload.sa4_sarw_inner_current
    :field sa5_sarw_outer_current: ax25_frame.payload.sa5_sarw_outer_current
    :field sa1_salw_inner_temperature: ax25_frame.payload.sa1_salw_inner_temperature
    :field sa2_salw_outer_temperature: ax25_frame.payload.sa2_salw_outer_temperature
    :field sa4_sarw_inner_temperature: ax25_frame.payload.sa4_sarw_inner_temperature
    :field sa5_sarw_outer_temperature: ax25_frame.payload.sa5_sarw_outer_temperature
    :field curr_boot_image: ax25_frame.payload.curr_boot_image
    :field image_valid: ax25_frame.payload.image_valid
    :field image_priority_0: ax25_frame.payload.image_priority_0
    :field image_priority_1: ax25_frame.payload.image_priority_1
    :field image_priority_2: ax25_frame.payload.image_priority_2
    :field image_is_stable: ax25_frame.payload.image_is_stable
    :field adc_enable: ax25_frame.payload.adc_enable
    :field last_reset_cause: ax25_frame.payload.last_reset_cause
    :field last_boot_count: ax25_frame.payload.last_boot_count
    :field version: ax25_frame.payload.version
    :field interface_baud_rate: ax25_frame.payload.interface_baud_rate
    :field rx_rf_baud_rate: ax25_frame.payload.rx_rf_baud_rate
    :field rx_modulation: ax25_frame.payload.rx_modulation
    :field rx_frequency: ax25_frame.payload.rx_frequency
    :field tx_power_amp_level: ax25_frame.payload.tx_power_amp_level
    :field tx_rf_baud_rate: ax25_frame.payload.tx_rf_baud_rate
    :field tx_modulation: ax25_frame.payload.tx_modulation
    :field tx_frequency: ax25_frame.payload.tx_frequency
    :field source_callsign_byte_0: ax25_frame.payload.source_callsign_byte_0
    :field source_callsign_byte_1: ax25_frame.payload.source_callsign_byte_1
    :field source_callsign_byte_2: ax25_frame.payload.source_callsign_byte_2
    :field source_callsign_byte_3: ax25_frame.payload.source_callsign_byte_3
    :field source_callsign_byte_4: ax25_frame.payload.source_callsign_byte_4
    :field source_callsign_byte_5: ax25_frame.payload.source_callsign_byte_5
    :field destination_callsign_byte_0: ax25_frame.payload.destination_callsign_byte_0
    :field destination_callsign_byte_1: ax25_frame.payload.destination_callsign_byte_1
    :field destination_callsign_byte_2: ax25_frame.payload.destination_callsign_byte_2
    :field destination_callsign_byte_3: ax25_frame.payload.destination_callsign_byte_3
    :field destination_callsign_byte_4: ax25_frame.payload.destination_callsign_byte_4
    :field destination_callsign_byte_5: ax25_frame.payload.destination_callsign_byte_5
    :field rssi: ax25_frame.payload.rssi
    :field vbat_obc_voltage: ax25_frame.payload.vbat_obc_voltage
    :field vbat_obc_current: ax25_frame.payload.vbat_obc_current
    :field vbat_plat_voltage: ax25_frame.payload.vbat_plat_voltage
    :field v3_3_plat_voltage: ax25_frame.payload.v3_3_plat_voltage
    :field v1_2_obc_voltage: ax25_frame.payload.v1_2_obc_voltage
    :field obc_temperature_1: ax25_frame.payload.obc_temperature_1
    :field v3_3_obc_voltage: ax25_frame.payload.v3_3_obc_voltage
    :field v3_3_obc_current: ax25_frame.payload.v3_3_obc_current
    :field v3_3_memory_voltage: ax25_frame.payload.v3_3_memory_voltage
    :field v3_3_memory_current: ax25_frame.payload.v3_3_memory_current
    :field vbat_periph_current: ax25_frame.payload.vbat_periph_current
    :field v3_3_periph_current: ax25_frame.payload.v3_3_periph_current
    :field v2_5_periph_current: ax25_frame.payload.v2_5_periph_current
    :field obc_temperature_2: ax25_frame.payload.obc_temperature_2
    :field obc_temperature_3: ax25_frame.payload.obc_temperature_3
    :field v3_3_gps_voltage: ax25_frame.payload.v3_3_gps_voltage
    :field v3_3_gps_current: ax25_frame.payload.v3_3_gps_current
    :field v2_5_obc_voltage: ax25_frame.payload.v2_5_obc_voltage
    :field v2_5_periph_voltage: ax25_frame.payload.v2_5_periph_voltage
    :field vbat_periph_voltage: ax25_frame.payload.vbat_periph_voltage
    :field v3_3_periph_voltage: ax25_frame.payload.v3_3_periph_voltage
    :field system_mode: ax25_frame.payload.system_mode
    :field startup_mode: ax25_frame.payload.startup_mode
    :field pass_in_progress: ax25_frame.payload.pass_in_progress
    :field digital_bus_voltage: ax25_frame.payload.digital_bus_voltage
    :field wheel_bus_voltage: ax25_frame.payload.wheel_bus_voltage
    :field rod_bus_voltage: ax25_frame.payload.rod_bus_voltage
    :field wheel_1_speed: ax25_frame.payload.wheel_1_speed
    :field wheel_2_speed: ax25_frame.payload.wheel_2_speed
    :field wheel_3_speed: ax25_frame.payload.wheel_3_speed
    :field adcs_mode: ax25_frame.payload.adcs_mode
    :field wheel_1_current: ax25_frame.payload.wheel_1_current
    :field wheel_2_current: ax25_frame.payload.wheel_2_current
    :field wheel_3_current: ax25_frame.payload.wheel_3_current
    :field imu_temp: ax25_frame.payload.imu_temp
    :field wheel_1_temperature: ax25_frame.payload.wheel_1_temperature
    :field wheel_2_temperature: ax25_frame.payload.wheel_2_temperature
    :field wheel_3_temperature: ax25_frame.payload.wheel_3_temperature
    :field crc: ax25_frame.payload.crc
    :field last_uptime: ax25_frame.payload.last_uptime
    :field time_fine_seconds: ax25_frame.payload.time_fine_seconds
    :field time_fine_fractional_seconds: ax25_frame.payload.time_fine_fractional_seconds
    :field time_onboard: ax25_frame.payload.time_onboard
    :field uptime: ax25_frame.payload.uptime
    :field mode_checkpoint_time: ax25_frame.payload.mode_checkpoint_time
    :field gps_time: ax25_frame.payload.gps_time
    :field tai_seconds: ax25_frame.payload.tai_seconds
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Real.Ax25Frame(self._io, self, self._root)

    class HealthBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bat_charging_status = self._io.read_bits_int_be(1) != 0
            self.v5_bat_current = self._io.read_bits_int_be(10)
            self.v3_3_bat_current = self._io.read_bits_int_be(10)
            self.vbat_bat_current = self._io.read_bits_int_be(10)
            self.v3_3_bat_voltage = self._io.read_bits_int_be(10)
            self.v5_bat_voltage = self._io.read_bits_int_be(10)
            self.vbat_bat_voltage = self._io.read_bits_int_be(10)
            self.bat_board_temperature = self._io.read_bits_int_be(10)
            self.bat_cell_1_temperature = self._io.read_bits_int_be(10)
            self.bat_cell_2_temperature = self._io.read_bits_int_be(10)
            self.bat_cell_3_temperature = self._io.read_bits_int_be(10)
            self.bat_cell_4_temperature = self._io.read_bits_int_be(10)
            self.battery_heater_status_0 = self._io.read_bits_int_be(10)
            self.battery_heater_status_1 = self._io.read_bits_int_be(10)
            self.battery_heater_status_2 = self._io.read_bits_int_be(10)
            self.battery_heater_status_3 = self._io.read_bits_int_be(10)
            self.vbat_eps_voltage = self._io.read_bits_int_be(10)
            self.v3_3_eps_voltage = self._io.read_bits_int_be(10)
            self.v5_eps_voltage = self._io.read_bits_int_be(10)
            self.v12_eps_voltage = self._io.read_bits_int_be(10)
            self.vbat_eps_current = self._io.read_bits_int_be(10)
            self.v3_3_eps_current = self._io.read_bits_int_be(10)
            self.v5_eps_current = self._io.read_bits_int_be(10)
            self.v12_eps_current = self._io.read_bits_int_be(10)
            self.sw1_unused_voltage = self._io.read_bits_int_be(10)
            self.sw2_unused_voltage = self._io.read_bits_int_be(10)
            self.sw3_instrument_voltage = self._io.read_bits_int_be(10)
            self.sw4_li2_voltage = self._io.read_bits_int_be(10)
            self.sw5_uhf_temp_sensor_voltage = self._io.read_bits_int_be(10)
            self.sw6_unused_voltage = self._io.read_bits_int_be(10)
            self.sw7_unused_voltage = self._io.read_bits_int_be(10)
            self.sw8_xact_serial_voltage = self._io.read_bits_int_be(10)
            self.sw9_gps_voltage = self._io.read_bits_int_be(10)
            self.sw10_instrument_lvds_voltage = self._io.read_bits_int_be(10)
            self.sw1_unused_current = self._io.read_bits_int_be(10)
            self.sw2_unused_current = self._io.read_bits_int_be(10)
            self.sw3_instrument_current = self._io.read_bits_int_be(10)
            self.sw4_li2_current = self._io.read_bits_int_be(10)
            self.sw5_uhf_temp_sensor_current = self._io.read_bits_int_be(10)
            self.sw6_unused_current = self._io.read_bits_int_be(10)
            self.sw7_unused_current = self._io.read_bits_int_be(10)
            self.sw8_xact_serial_current = self._io.read_bits_int_be(10)
            self.sw9_gps_current = self._io.read_bits_int_be(10)
            self.sw10_instrument_lvds_current = self._io.read_bits_int_be(10)
            self.eps_mb_temperature = self._io.read_bits_int_be(10)
            self.eps_db_temperature = self._io.read_bits_int_be(10)
            self.sa1_salw_inner_voltage = self._io.read_bits_int_be(10)
            self.sa2_salw_outer_voltage = self._io.read_bits_int_be(10)
            self.sa4_sarw_inner_voltage = self._io.read_bits_int_be(10)
            self.sa5_sarw_outer_voltage = self._io.read_bits_int_be(10)
            self.sa1_salw_inner_current = self._io.read_bits_int_be(10)
            self.sa2_salw_outer_current = self._io.read_bits_int_be(10)
            self.sa4_sarw_inner_current = self._io.read_bits_int_be(10)
            self.sa5_sarw_outer_current = self._io.read_bits_int_be(10)
            self.sa1_salw_inner_temperature = self._io.read_bits_int_be(10)
            self.sa2_salw_outer_temperature = self._io.read_bits_int_be(10)
            self.sa4_sarw_inner_temperature = self._io.read_bits_int_be(10)
            self.sa5_sarw_outer_temperature = self._io.read_bits_int_be(10)
            self.curr_boot_image = self._io.read_bits_int_be(8)
            self.image_valid = self._io.read_bits_int_be(3)
            self.image_priority_0 = self._io.read_bits_int_be(32)
            self.image_priority_1 = self._io.read_bits_int_be(32)
            self.image_priority_2 = self._io.read_bits_int_be(32)
            self.image_is_stable = self._io.read_bits_int_be(3)
            self.adc_enable = self._io.read_bits_int_be(1) != 0
            self.last_reset_cause = self._io.read_bits_int_be(2)
            self.last_boot_count = self._io.read_bits_int_be(32)
            self.version = self._io.read_bits_int_be(32)
            self.interface_baud_rate = self._io.read_bits_int_be(3)
            self.rx_rf_baud_rate = self._io.read_bits_int_be(2)
            self.rx_modulation = self._io.read_bits_int_be(2)
            self.rx_frequency = self._io.read_bits_int_be(32)
            self.tx_power_amp_level = self._io.read_bits_int_be(8)
            self.tx_rf_baud_rate = self._io.read_bits_int_be(2)
            self.tx_modulation = self._io.read_bits_int_be(2)
            self.tx_frequency = self._io.read_bits_int_be(32)
            self.source_callsign_byte_0 = self._io.read_bits_int_be(8)
            self.source_callsign_byte_1 = self._io.read_bits_int_be(8)
            self.source_callsign_byte_2 = self._io.read_bits_int_be(8)
            self.source_callsign_byte_3 = self._io.read_bits_int_be(8)
            self.source_callsign_byte_4 = self._io.read_bits_int_be(8)
            self.source_callsign_byte_5 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_0 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_1 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_2 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_3 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_4 = self._io.read_bits_int_be(8)
            self.destination_callsign_byte_5 = self._io.read_bits_int_be(8)
            self.rssi = self._io.read_bits_int_be(8)
            self.vbat_obc_voltage = self._io.read_bits_int_be(12)
            self.vbat_obc_current = self._io.read_bits_int_be(12)
            self.vbat_plat_voltage = self._io.read_bits_int_be(12)
            self.unused_a = self._io.read_bits_int_be(12)
            self.v3_3_plat_voltage = self._io.read_bits_int_be(12)
            self.v1_2_obc_voltage = self._io.read_bits_int_be(12)
            self.unused_b = self._io.read_bits_int_be(12)
            self.obc_temperature_1 = self._io.read_bits_int_be(12)
            self.v3_3_obc_voltage = self._io.read_bits_int_be(12)
            self.v3_3_obc_current = self._io.read_bits_int_be(12)
            self.v3_3_memory_voltage = self._io.read_bits_int_be(12)
            self.v3_3_memory_current = self._io.read_bits_int_be(12)
            self.vbat_periph_current = self._io.read_bits_int_be(12)
            self.v3_3_periph_current = self._io.read_bits_int_be(12)
            self.v2_5_periph_current = self._io.read_bits_int_be(12)
            self.obc_temperature_2 = self._io.read_bits_int_be(12)
            self.obc_temperature_3 = self._io.read_bits_int_be(12)
            self.v3_3_gps_voltage = self._io.read_bits_int_be(12)
            self.v3_3_gps_current = self._io.read_bits_int_be(12)
            self.v2_5_obc_voltage = self._io.read_bits_int_be(12)
            self.v2_5_periph_voltage = self._io.read_bits_int_be(12)
            self.vbat_periph_voltage = self._io.read_bits_int_be(12)
            self.v3_3_periph_voltage = self._io.read_bits_int_be(12)
            self.unused_c = self._io.read_bits_int_be(12)
            self.system_mode = self._io.read_bits_int_be(3)
            self.startup_mode = self._io.read_bits_int_be(3)
            self.pass_in_progress = self._io.read_bits_int_be(1) != 0
            self.digital_bus_voltage = self._io.read_bits_int_be(16)
            self.wheel_bus_voltage = self._io.read_bits_int_be(16)
            self.rod_bus_voltage = self._io.read_bits_int_be(16)
            self.wheel_1_speed = self._io.read_bits_int_be(16)
            self.wheel_2_speed = self._io.read_bits_int_be(16)
            self.wheel_3_speed = self._io.read_bits_int_be(16)
            self.adcs_mode = self._io.read_bits_int_be(8)
            self.wheel_1_current = self._io.read_bits_int_be(16)
            self.wheel_2_current = self._io.read_bits_int_be(16)
            self.wheel_3_current = self._io.read_bits_int_be(16)
            self.imu_temp = self._io.read_bits_int_be(16)
            self.wheel_1_temperature = self._io.read_bits_int_be(16)
            self.wheel_2_temperature = self._io.read_bits_int_be(16)
            self.wheel_3_temperature = self._io.read_bits_int_be(16)
            self.unused_j = self._io.read_bits_int_be(16)
            self.crc = self._io.read_bits_int_be(16)


    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Real.Ax25Header(self._io, self, self._root)
            self.ccsds_header = Real.CcsdsHeader(self._io, self, self._root)
            self.real_header = Real.RealHeader(self._io, self, self._root)
            _on = self.ccsds_header.length
            if _on == 192:
                self.payload = Real.HealthBeacon(self._io, self, self._root)
            elif _on == 45:
                self.payload = Real.TimeBeacon(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Real.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Real.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Real.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Real.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u2be()


    class CcsdsHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ccsds_header_first = self._io.read_u4be()
            self.length = self._io.read_u2be()
            self.ccsds_header_second = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not self.callsign == u"WR9XTX":
                raise kaitaistruct.ValidationNotEqualError(u"WR9XTX", self.callsign, self._io, u"/types/callsign/seq/0")


    class RealSyncword(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.syncword = (self._io.read_bytes(4)).decode(u"ASCII")
            if not self.syncword == u"REAL":
                raise kaitaistruct.ValidationNotEqualError(u"REAL", self.syncword, self._io, u"/types/real_syncword/seq/0")


    class RealHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.service_type = self._io.read_u1()
            self.service_sub_type = self._io.read_u1()
            self.structure_id = self._io.read_u1()
            self.syncword = Real.RealSyncword(self._io, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return getattr(self, '_m_ssid', None)


    class TimeBeacon(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.last_uptime = self._io.read_u4be()
            self.time_fine_seconds = self._io.read_u4be()
            self.time_fine_fractional_seconds = self._io.read_u4be()
            self.time_onboard = self._io.read_u4be()
            self.uptime = self._io.read_u4be()
            self.mode_checkpoint_time = self._io.read_u4be()
            self.gps_time = self._io.read_u4be()
            self.tai_seconds = self._io.read_f8be()
            self.crc = self._io.read_bits_int_be(16)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Real.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class No44(KaitaiStruct):
    """:field counter: no44.type_check.counter
    :field current_plus_x: no44.type_check.current_plus_x
    :field current_plus_z: no44.type_check.current_plus_z
    :field current_plus_y: no44.type_check.current_plus_y
    :field current_minus_x: no44.type_check.current_minus_x
    :field temp_plus_y: no44.type_check.temp_plus_y
    :field temp_batt_a: no44.type_check.temp_batt_a
    :field temp_xmit_a: no44.type_check.temp_xmit_a
    :field temp_plus_z: no44.type_check.temp_plus_z
    :field temp_plus_x: no44.type_check.temp_plus_x
    :field temp_stack_a: no44.type_check.temp_stack_a
    :field current_minus_y: no44.type_check.current_minus_y
    :field current_batt_a: no44.type_check.current_batt_a
    :field a_batt_a_volt: no44.type_check.a_batt_a_volt
    :field a_batt_b_volt: no44.type_check.a_batt_b_volt
    :field power_out_a: no44.type_check.power_out_a
    :field eight_v_reg_a: no44.type_check.eight_v_reg_a
    :field current_minus_x: no44.type_check.current_minus_x
    :field current_minus_z: no44.type_check.current_minus_z
    :field current_minus_y: no44.type_check.current_minus_y
    :field current_plus_x: no44.type_check.current_plus_x
    :field temp_minus_y: no44.type_check.temp_minus_y
    :field temp_batt_b: no44.type_check.temp_batt_b
    :field temp_xmit_b: no44.type_check.temp_xmit_b
    :field temp_minus_z: no44.type_check.temp_minus_z
    :field temp_minus_x: no44.type_check.temp_minus_x
    :field temp_stack_b: no44.type_check.temp_stack_b
    :field current_plus_y: no44.type_check.current_plus_y
    :field current_batt_b: no44.type_check.current_batt_b
    :field b_batt_a_volt: no44.type_check.b_batt_a_volt
    :field b_batt_b_volt: no44.type_check.b_batt_b_volt
    :field power_out_b: no44.type_check.power_out_b
    :field eight_v_reg_b: no44.type_check.eight_v_reg_b
    :field monitor: no44.type_check.monitor
    :field dest_callsign: no44.type_check.ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: no44.type_check.ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: no44.type_check.ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: no44.type_check.ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_instance___callsign: no44.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_callsign_raw.callsign_ror.callsign
    :field rpt_instance___ssid: no44.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.ssid
    :field rpt_instance___hbit: no44.type_check.ax25_frame.ax25_header.repeater.rpt_instance.___.rpt_ssid_raw.hbit
    :field ctl: no44.type_check.ax25_frame.ax25_header.ctl
    :field pid: no44.type_check.ax25_frame.payload.pid
    :field monitor: no44.type_check.ax25_frame.payload.ax25_info.data_monitor
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.no44 = No44.No44T(self._io, self, self._root)

    class No44T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.check
            if _on == 2925953672:
                self.type_check = No44.SideA(self._io, self, self._root)
            elif _on == 2693179010:
                self.type_check = No44.SideB(self._io, self, self._root)
            else:
                self.type_check = No44.Aprs(self._io, self, self._root)

        @property
        def check(self):
            if hasattr(self, '_m_check'):
                return self._m_check

            _pos = self._io.pos()
            self._io.seek(7)
            self._m_check = self._io.read_u4be()
            self._io.seek(_pos)
            return getattr(self, '_m_check', None)


    class SideA(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header_0 = self._io.read_u8be()
            if not  ((self.header_0 == 9550589474490835118)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_0, self._io, u"/types/side_a/seq/0")
            self.header_1 = self._io.read_u8be()
            if not  ((self.header_1 == 7386616552107452046)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_1, self._io, u"/types/side_a/seq/1")
            self.header_2 = self._io.read_u8be()
            if not  ((self.header_2 == 9414927030128210004)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_2, self._io, u"/types/side_a/seq/2")
            self.header_3 = self._io.read_u1()
            if not  ((self.header_3 == 35)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_3, self._io, u"/types/side_a/seq/3")
            self.counter = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_1 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_2 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_3 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_4 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.five_v_reference = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.ones = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.cycle_count = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.last_number = (self._io.read_bytes_term(13, False, True, True)).decode(u"ASCII")

        @property
        def current_minus_x(self):
            if hasattr(self, '_m_current_minus_x'):
                return self._m_current_minus_x

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_minus_x = (((0.0024 * (int(self.value_4) * int(self.value_4))) + (0.414 * int(self.value_4))) - 25.3)

            return getattr(self, '_m_current_minus_x', None)

        @property
        def a_batt_a_volt(self):
            if hasattr(self, '_m_a_batt_a_volt'):
                return self._m_a_batt_a_volt

            if (self.cycle_count)[2:4] == u"11":
                self._m_a_batt_a_volt = (0.0984 * int(self.value_1))

            return getattr(self, '_m_a_batt_a_volt', None)

        @property
        def current_plus_y(self):
            if hasattr(self, '_m_current_plus_y'):
                return self._m_current_plus_y

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_plus_y = (((0.0031 * (int(self.value_3) * int(self.value_3))) + (0.241 * int(self.value_3))) - 25.3)

            return getattr(self, '_m_current_plus_y', None)

        @property
        def current_plus_z(self):
            if hasattr(self, '_m_current_plus_z'):
                return self._m_current_plus_z

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_plus_z = (((0.0048 * (int(self.value_2) * int(self.value_2))) + (0.75 * int(self.value_2))) - 54.6)

            return getattr(self, '_m_current_plus_z', None)

        @property
        def monitor(self):
            if hasattr(self, '_m_monitor'):
                return self._m_monitor

            self._m_monitor = u"T#" + self.counter + u"," + self.value_1 + u"," + self.value_2 + u"," + self.value_3 + u"," + self.value_4 + u"," + self.five_v_reference + u"," + self.ones + u"," + self.cycle_count + u"," + self.last_number
            return getattr(self, '_m_monitor', None)

        @property
        def power_out_a(self):
            if hasattr(self, '_m_power_out_a'):
                return self._m_power_out_a

            if (self.cycle_count)[2:4] == u"11":
                self._m_power_out_a = (0.0311 * int(self.value_3))

            return getattr(self, '_m_power_out_a', None)

        @property
        def temp_plus_z(self):
            if hasattr(self, '_m_temp_plus_z'):
                return self._m_temp_plus_z

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_plus_z = ((0.3414 * int(self.value_4)) - 19.71)

            return getattr(self, '_m_temp_plus_z', None)

        @property
        def temp_xmit_a(self):
            if hasattr(self, '_m_temp_xmit_a'):
                return self._m_temp_xmit_a

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_xmit_a = ((0.3414 * int(self.value_3)) - 19.71)

            return getattr(self, '_m_temp_xmit_a', None)

        @property
        def temp_stack_a(self):
            if hasattr(self, '_m_temp_stack_a'):
                return self._m_temp_stack_a

            if (self.cycle_count)[2:4] == u"10":
                self._m_temp_stack_a = ((0.3414 * int(self.value_2)) - 19.71)

            return getattr(self, '_m_temp_stack_a', None)

        @property
        def temp_plus_y(self):
            if hasattr(self, '_m_temp_plus_y'):
                return self._m_temp_plus_y

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_plus_y = ((0.3414 * int(self.value_1)) - 19.71)

            return getattr(self, '_m_temp_plus_y', None)

        @property
        def current_batt_a(self):
            if hasattr(self, '_m_current_batt_a'):
                return self._m_current_batt_a

            if (self.cycle_count)[2:4] == u"10":
                self._m_current_batt_a = ((((-0.00004 * ((int(self.value_4) * int(self.value_4)) * int(self.value_4))) + (0.0114 * (int(self.value_4) * int(self.value_4)))) - (2.56 * int(self.value_4))) + 252)

            return getattr(self, '_m_current_batt_a', None)

        @property
        def eight_v_reg_a(self):
            if hasattr(self, '_m_eight_v_reg_a'):
                return self._m_eight_v_reg_a

            if (self.cycle_count)[2:4] == u"11":
                self._m_eight_v_reg_a = (0.0356 * int(self.value_4))

            return getattr(self, '_m_eight_v_reg_a', None)

        @property
        def current_minus_y(self):
            if hasattr(self, '_m_current_minus_y'):
                return self._m_current_minus_y

            if (self.cycle_count)[2:4] == u"10":
                self._m_current_minus_y = (((0.0037 * (int(self.value_3) * int(self.value_3))) + (0.0264 * int(self.value_3))) - 18.5)

            return getattr(self, '_m_current_minus_y', None)

        @property
        def a_batt_b_volt(self):
            if hasattr(self, '_m_a_batt_b_volt'):
                return self._m_a_batt_b_volt

            if (self.cycle_count)[2:4] == u"11":
                self._m_a_batt_b_volt = (0.09826 * int(self.value_2))

            return getattr(self, '_m_a_batt_b_volt', None)

        @property
        def temp_batt_a(self):
            if hasattr(self, '_m_temp_batt_a'):
                return self._m_temp_batt_a

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_batt_a = ((0.3414 * int(self.value_2)) - 19.71)

            return getattr(self, '_m_temp_batt_a', None)

        @property
        def temp_plus_x(self):
            if hasattr(self, '_m_temp_plus_x'):
                return self._m_temp_plus_x

            if (self.cycle_count)[2:4] == u"10":
                self._m_temp_plus_x = ((0.3414 * int(self.value_1)) - 19.71)

            return getattr(self, '_m_temp_plus_x', None)

        @property
        def current_plus_x(self):
            if hasattr(self, '_m_current_plus_x'):
                return self._m_current_plus_x

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_plus_x = (((0.0012 * (int(self.value_1) * int(self.value_1))) + (0.646 * int(self.value_1))) - 25.96)

            return getattr(self, '_m_current_plus_x', None)


    class SideB(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header_0 = self._io.read_u8be()
            if not  ((self.header_0 == 9550589474490835104)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_0, self._io, u"/types/side_b/seq/0")
            self.header_1 = self._io.read_u8be()
            if not  ((self.header_1 == 9702586106363946638)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_1, self._io, u"/types/side_b/seq/1")
            self.header_2 = self._io.read_u8be()
            if not  ((self.header_2 == 9414927030128210004)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_2, self._io, u"/types/side_b/seq/2")
            self.header_3 = self._io.read_u1()
            if not  ((self.header_3 == 35)) :
                raise kaitaistruct.ValidationNotAnyOfError(self.header_3, self._io, u"/types/side_b/seq/3")
            self.counter = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_1 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_2 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_3 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.value_4 = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.five_v_reference = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.ones = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.cycle_count = (self._io.read_bytes_term(44, False, True, True)).decode(u"ASCII")
            self.last_number = (self._io.read_bytes_term(13, False, True, True)).decode(u"ASCII")

        @property
        def current_minus_x(self):
            if hasattr(self, '_m_current_minus_x'):
                return self._m_current_minus_x

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_minus_x = (((0.0034 * (int(self.value_1) * int(self.value_1))) + (0.2284 * int(self.value_1))) - 26.6)

            return getattr(self, '_m_current_minus_x', None)

        @property
        def b_batt_a_volt(self):
            if hasattr(self, '_m_b_batt_a_volt'):
                return self._m_b_batt_a_volt

            if (self.cycle_count)[2:4] == u"11":
                self._m_b_batt_a_volt = (0.09774 * int(self.value_1))

            return getattr(self, '_m_b_batt_a_volt', None)

        @property
        def current_plus_y(self):
            if hasattr(self, '_m_current_plus_y'):
                return self._m_current_plus_y

            if (self.cycle_count)[2:4] == u"10":
                self._m_current_plus_y = (((0.0038 * (int(self.value_3) * int(self.value_3))) + (0.0084 * int(self.value_3))) - 19.8)

            return getattr(self, '_m_current_plus_y', None)

        @property
        def temp_stack_b(self):
            if hasattr(self, '_m_temp_stack_b'):
                return self._m_temp_stack_b

            if (self.cycle_count)[2:4] == u"10":
                self._m_temp_stack_b = ((0.3414 * int(self.value_2)) - 19.71)

            return getattr(self, '_m_temp_stack_b', None)

        @property
        def current_minus_z(self):
            if hasattr(self, '_m_current_minus_z'):
                return self._m_current_minus_z

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_minus_z = (((0.0096 * (int(self.value_2) * int(self.value_2))) + (0.864 * int(self.value_2))) - 53.8)

            return getattr(self, '_m_current_minus_z', None)

        @property
        def temp_minus_x(self):
            if hasattr(self, '_m_temp_minus_x'):
                return self._m_temp_minus_x

            if (self.cycle_count)[2:4] == u"10":
                self._m_temp_minus_x = ((0.3414 * int(self.value_1)) - 19.71)

            return getattr(self, '_m_temp_minus_x', None)

        @property
        def b_batt_b_volt(self):
            if hasattr(self, '_m_b_batt_b_volt'):
                return self._m_b_batt_b_volt

            if (self.cycle_count)[2:4] == u"11":
                self._m_b_batt_b_volt = (0.09457 * int(self.value_2))

            return getattr(self, '_m_b_batt_b_volt', None)

        @property
        def power_out_b(self):
            if hasattr(self, '_m_power_out_b'):
                return self._m_power_out_b

            if (self.cycle_count)[2:4] == u"11":
                self._m_power_out_b = (0.0223 * int(self.value_3))

            return getattr(self, '_m_power_out_b', None)

        @property
        def temp_batt_b(self):
            if hasattr(self, '_m_temp_batt_b'):
                return self._m_temp_batt_b

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_batt_b = ((0.3414 * int(self.value_2)) - 19.71)

            return getattr(self, '_m_temp_batt_b', None)

        @property
        def temp_minus_z(self):
            if hasattr(self, '_m_temp_minus_z'):
                return self._m_temp_minus_z

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_minus_z = ((0.3414 * int(self.value_4)) - 19.71)

            return getattr(self, '_m_temp_minus_z', None)

        @property
        def monitor(self):
            if hasattr(self, '_m_monitor'):
                return self._m_monitor

            self._m_monitor = u"T#" + self.counter + u"," + self.value_1 + u"," + self.value_2 + u"," + self.value_3 + u"," + self.value_4 + u"," + self.five_v_reference + u"," + self.ones + u"," + self.cycle_count + u"," + self.last_number
            return getattr(self, '_m_monitor', None)

        @property
        def temp_xmit_b(self):
            if hasattr(self, '_m_temp_xmit_b'):
                return self._m_temp_xmit_b

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_xmit_b = ((0.3414 * int(self.value_3)) - 19.71)

            return getattr(self, '_m_temp_xmit_b', None)

        @property
        def current_minus_y(self):
            if hasattr(self, '_m_current_minus_y'):
                return self._m_current_minus_y

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_minus_y = (((0.0023 * (int(self.value_3) * int(self.value_3))) + (0.473 * int(self.value_3))) - 23.2)

            return getattr(self, '_m_current_minus_y', None)

        @property
        def eight_v_reg_b(self):
            if hasattr(self, '_m_eight_v_reg_b'):
                return self._m_eight_v_reg_b

            if (self.cycle_count)[2:4] == u"11":
                self._m_eight_v_reg_b = (0.0351 * int(self.value_4))

            return getattr(self, '_m_eight_v_reg_b', None)

        @property
        def current_plus_x(self):
            if hasattr(self, '_m_current_plus_x'):
                return self._m_current_plus_x

            if (self.cycle_count)[2:4] == u"00":
                self._m_current_plus_x = (((0.003 * (int(self.value_4) * int(self.value_4))) + (0.4 * int(self.value_4))) - 26.6)

            return getattr(self, '_m_current_plus_x', None)

        @property
        def current_batt_b(self):
            if hasattr(self, '_m_current_batt_b'):
                return self._m_current_batt_b

            if (self.cycle_count)[2:4] == u"10":
                self._m_current_batt_b = ((((-0.00004 * ((int(self.value_4) * int(self.value_4)) * int(self.value_4))) + (0.0158 * (int(self.value_4) * int(self.value_4)))) - (3.32 * int(self.value_4))) + 259)

            return getattr(self, '_m_current_batt_b', None)

        @property
        def temp_minus_y(self):
            if hasattr(self, '_m_temp_minus_y'):
                return self._m_temp_minus_y

            if (self.cycle_count)[2:4] == u"01":
                self._m_temp_minus_y = ((0.3414 * int(self.value_1)) - 19.71)

            return getattr(self, '_m_temp_minus_y', None)


    class Aprs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_frame = No44.Aprs.Ax25Frame(self._io, self, self._root)

        class Ax25Frame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ax25_header = No44.Aprs.Ax25Header(self._io, self, self._root)
                _on = (self.ax25_header.ctl & 19)
                if _on == 0:
                    self.payload = No44.Aprs.IFrame(self._io, self, self._root)
                elif _on == 3:
                    self.payload = No44.Aprs.UiFrame(self._io, self, self._root)
                elif _on == 19:
                    self.payload = No44.Aprs.UiFrame(self._io, self, self._root)
                elif _on == 16:
                    self.payload = No44.Aprs.IFrame(self._io, self, self._root)
                elif _on == 18:
                    self.payload = No44.Aprs.IFrame(self._io, self, self._root)
                elif _on == 2:
                    self.payload = No44.Aprs.IFrame(self._io, self, self._root)


        class Ax25Header(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.dest_callsign_raw = No44.Aprs.CallsignRaw(self._io, self, self._root)
                self.dest_ssid_raw = No44.Aprs.SsidMask(self._io, self, self._root)
                self.src_callsign_raw = No44.Aprs.CallsignRaw(self._io, self, self._root)
                self.src_ssid_raw = No44.Aprs.SsidMask(self._io, self, self._root)
                if (self.src_ssid_raw.ssid_mask & 1) == 0:
                    self.repeater = No44.Aprs.Repeater(self._io, self, self._root)

                self.ctl = self._io.read_u1()


        class UiFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = No44.Aprs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class Callsign(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


        class IFrame(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pid = self._io.read_u1()
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = No44.Aprs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


        class SsidMask(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.ssid_mask = self._io.read_u1()

            @property
            def ssid(self):
                if hasattr(self, '_m_ssid'):
                    return self._m_ssid

                self._m_ssid = ((self.ssid_mask & 31) >> 1)
                return getattr(self, '_m_ssid', None)

            @property
            def hbit(self):
                if hasattr(self, '_m_hbit'):
                    return self._m_hbit

                self._m_hbit = ((self.ssid_mask & 128) >> 7)
                return getattr(self, '_m_hbit', None)


        class Repeaters(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_callsign_raw = No44.Aprs.CallsignRaw(self._io, self, self._root)
                self.rpt_ssid_raw = No44.Aprs.SsidMask(self._io, self, self._root)


        class Repeater(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rpt_instance = []
                i = 0
                while True:
                    _ = No44.Aprs.Repeaters(self._io, self, self._root)
                    self.rpt_instance.append(_)
                    if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                        break
                    i += 1


        class CallsignRaw(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self._raw__raw_callsign_ror = self._io.read_bytes(6)
                self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
                _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
                self.callsign_ror = No44.Aprs.Callsign(_io__raw_callsign_ror, self, self._root)


        class Ax25InfoData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.data_monitor = (self._io.read_bytes_full()).decode(u"utf-8")







# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Amusat(KaitaiStruct):
    """:field flag1: flag1
    :field dest_address: ax25_header.dest_address
    :field dest_ssid: ax25_header.dest_ssid
    :field src_address: ax25_header.src_address
    :field src_ssid: ax25_header.src_ssid
    :field control_id: ax25_header.control_id
    :field pid: ax25_header.pid
    :field packet_type: information_field.packet_type
    :field sensor1: information_field.telemetry_data_1.sensor1
    :field sensor2: information_field.telemetry_data_1.sensor2
    :field sensor3: information_field.telemetry_data_1.sensor3
    :field sensor4: information_field.telemetry_data_1.sensor4
    :field battery_voltage: information_field.telemetry_data_1.battery_voltage
    :field battery_temperature: information_field.telemetry_data_1.battery_temperature
    :field battery_soc: information_field.telemetry_data_1.battery_soc
    :field battery_current: information_field.telemetry_data_1.battery_current
    :field temp_in: information_field.telemetry_data_1.temp_in
    :field temp_out: information_field.telemetry_data_1.temp_out
    :field temp_3: information_field.telemetry_data_1.temp_3
    :field pl_temp: information_field.telemetry_data_1.pl_temp
    :field rtc: information_field.telemetry_data_1.rtc
    :field temp_out_l: information_field.telemetry_data_1.temp_out_l
    :field temp_out_h: information_field.telemetry_data_1.temp_out_h
    :field acceleration_x: information_field.telemetry_data_1.acceleration_x
    :field acceleration_y: information_field.telemetry_data_1.acceleration_y
    :field acceleration_z: information_field.telemetry_data_1.acceleration_z
    :field magnetic_field_x: information_field.telemetry_data_1.magnetic_field_x
    :field magnetic_field_y: information_field.telemetry_data_1.magnetic_field_y
    :field magnetic_field_z: information_field.telemetry_data_1.magnetic_field_z
    :field angular_rate_x: information_field.telemetry_data_1.angular_rate_x
    :field angular_rate_y: information_field.telemetry_data_1.angular_rate_y
    :field angular_rate_z: information_field.telemetry_data_1.angular_rate_z
    :field velocity: information_field.telemetry_data_1.velocity
    :field latitude: information_field.telemetry_data_1.latitude
    :field longitude: information_field.telemetry_data_1.longitude
    :field gps_time: information_field.telemetry_data_1.gps_time
    :field solar_deployment_status: information_field.telemetry_data_1.solar_deployment_status
    :field eps_health_status: information_field.telemetry_data_1.eps_health_status
    :field adcs_health_status: information_field.telemetry_data_1.adcs_health_status
    :field payload_health_status: information_field.telemetry_data_1.payload_health_status
    :field com_health_status: information_field.telemetry_data_1.com_health_status
    :field battery1_health_status: information_field.telemetry_data_1.battery1_health_status
    :field battery2_health_status: information_field.telemetry_data_1.battery2_health_status
    :field image_number: information_field.image_data_0.image_number
    :field total_image_packets: information_field.image_data_0.total_image_packets
    :field image_packet_number: information_field.image_data_0.image_packet_number
    :field payload_array_size: information_field.image_data_0.payload_array_size
    :field payload_data: information_field.image_data_0.payload_data.b64encstring.image_data_b64_encoded
    :field fcs: fcs
    :field flag2: flag2
    
    .. seealso::
       Source - https://amu-sat.github.io/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.flag1 = self._io.read_u1()
        self.ax25_header = Amusat.Ax25HeaderStruct(self._io, self, self._root)
        self.information_field = Amusat.InformationFieldStruct(self._io, self, self._root)
        self.fcs = self._io.read_u2be()
        self.flag2 = self._io.read_u1()

    class InformationFieldStruct(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_type = self._io.read_u1()
            if self.packet_type == 1:
                self.telemetry_data_1 = Amusat.TelemetryDataStruct1(self._io, self, self._root)

            if self.packet_type == 0:
                self.image_data_0 = Amusat.ImageDataStruct0(self._io, self, self._root)



    class Base64string(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.image_data_b64_encoded = (self._io.read_bytes(236)).decode(u"UTF-8")


    class Base64(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_b64encstring = self._io.read_bytes(236)
            _process = satnogsdecoders.process.B64encode()
            self._raw_b64encstring = _process.decode(self._raw__raw_b64encstring)
            _io__raw_b64encstring = KaitaiStream(BytesIO(self._raw_b64encstring))
            self.b64encstring = Amusat.Base64string(_io__raw_b64encstring, self, self._root)


    class ImageDataStruct0(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.image_number = self._io.read_u1()
            self.total_image_packets = self._io.read_u8be()
            self.image_packet_number = self._io.read_u8be()
            self.payload_array_size = self._io.read_u1()
            self._raw_payload_data = self._io.read_bytes(236)
            _io__raw_payload_data = KaitaiStream(BytesIO(self._raw_payload_data))
            self.payload_data = Amusat.Base64(_io__raw_payload_data, self, self._root)


    class Ax25HeaderStruct(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_address = (self._io.read_bytes(7)).decode(u"ASCII")
            self.dest_ssid = self._io.read_u1()
            self.src_address = (self._io.read_bytes(7)).decode(u"ASCII")
            self.src_ssid = self._io.read_u1()
            self.control_id = self._io.read_u1()
            self.pid = self._io.read_u1()


    class TelemetryDataStruct1(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sensor1 = self._io.read_f4be()
            self.sensor2 = self._io.read_f4be()
            self.sensor3 = self._io.read_f4be()
            self.sensor4 = self._io.read_f4be()
            self.battery_voltage = self._io.read_f4be()
            self.battery_temperature = self._io.read_f4be()
            self.battery_soc = self._io.read_f4be()
            self.battery_current = self._io.read_f4be()
            self.temp_in = self._io.read_u1()
            self.temp_out = self._io.read_u1()
            self.temp_3 = self._io.read_u1()
            self.pl_temp = self._io.read_u1()
            self.rtc = self._io.read_u8be()
            self.temp_out_l = self._io.read_u1()
            self.temp_out_h = self._io.read_u1()
            self.acceleration_x = self._io.read_f4be()
            self.acceleration_y = self._io.read_f4be()
            self.acceleration_z = self._io.read_f4be()
            self.magnetic_field_x = self._io.read_f4be()
            self.magnetic_field_y = self._io.read_f4be()
            self.magnetic_field_z = self._io.read_f4be()
            self.angular_rate_x = self._io.read_f4be()
            self.angular_rate_y = self._io.read_f4be()
            self.angular_rate_z = self._io.read_f4be()
            self.velocity = self._io.read_f4be()
            self.latitude = self._io.read_f4be()
            self.longitude = self._io.read_f4be()
            self.gps_time = self._io.read_u8be()
            self.solar_deployment_status = self._io.read_bits_int_be(1) != 0
            self.eps_health_status = self._io.read_bits_int_be(1) != 0
            self.adcs_health_status = self._io.read_bits_int_be(1) != 0
            self.payload_health_status = self._io.read_bits_int_be(1) != 0
            self.com_health_status = self._io.read_bits_int_be(1) != 0
            self.battery1_health_status = self._io.read_bits_int_be(1) != 0
            self.battery2_health_status = self._io.read_bits_int_be(1) != 0






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Aausat4(KaitaiStruct):
    """:field data_plus_hmac_length: packet.data_length.data_plus_hmac_length
    :field frame_length: frame_length
    :field csp_hdr_crc: packet.csp_header.crc
    :field csp_hdr_rdp: packet.csp_header.rdp
    :field csp_hdr_xtea: packet.csp_header.xtea
    :field csp_hdr_hmac: packet.csp_header.hmac
    :field csp_hdr_src_port: packet.csp_header.src_port
    :field csp_hdr_dst_port: packet.csp_header.dst_port
    :field csp_hdr_destination: packet.csp_header.destination
    :field csp_hdr_source: packet.csp_header.source
    :field csp_hdr_priority: packet.csp_header.priority
    :field subsystems_valid: packet.data.beacon.valid
    :field eps_boot_count: packet.data.beacon.eps.boot_count
    :field eps_boot_cause: packet.data.beacon.eps.boot_cause
    :field eps_uptime: packet.data.beacon.eps.uptime
    :field eps_rt_clock: packet.data.beacon.eps.rt_clock
    :field eps_ping_status: packet.data.beacon.eps.ping_status
    :field eps_subsystem_selfstatus: packet.data.beacon.eps.subsystem_selfstatus
    :field eps_battery_voltage: packet.data.beacon.eps.battery_voltage
    :field eps_cell_diff: packet.data.beacon.eps.cell_diff
    :field eps_battery_current: packet.data.beacon.eps.battery_current
    :field eps_solar_power: packet.data.beacon.eps.solar_power
    :field eps_temp: packet.data.beacon.eps.temp
    :field eps_pa_temp: packet.data.beacon.eps.pa_temp
    :field eps_main_voltage: packet.data.beacon.eps.main_voltage
    :field com_boot_count: packet.data.beacon.com.boot_count
    :field com_boot_cause: packet.data.beacon.com.boot_cause
    
    .. seealso::
       Source - https://github.com/aausat/aausat4_beacon_parser/blob/master/beacon.py
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.packet = Aausat4.Aausat4DataT(self._io, self, self._root)

    class ComT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.boot_count_raw = self._io.read_u2be()

        @property
        def boot_count(self):
            if hasattr(self, '_m_boot_count'):
                return self._m_boot_count

            self._m_boot_count = (self.boot_count_raw & 8191)
            return getattr(self, '_m_boot_count', None)

        @property
        def boot_cause(self):
            if hasattr(self, '_m_boot_cause'):
                return self._m_boot_cause

            self._m_boot_cause = ((self.boot_count_raw & 57344) >> 13)
            return getattr(self, '_m_boot_cause', None)


    class Aausat4HmacT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.hmac = self._io.read_u2be()


    class EpsT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.boot_count_raw = self._io.read_u2be()
            self.uptime = self._io.read_u4be()
            self.rt_clock = self._io.read_u4be()
            self.ping_status = self._io.read_u1()
            self.subsystem_selfstatus = self._io.read_u2be()
            self.battery_voltage = self._io.read_u1()
            self.cell_diff = self._io.read_s1()
            self.battery_current = self._io.read_s1()
            self.solar_power = self._io.read_u1()
            self.temp = self._io.read_s1()
            self.pa_temp = self._io.read_s1()
            self.main_voltage = self._io.read_s1()

        @property
        def boot_count(self):
            if hasattr(self, '_m_boot_count'):
                return self._m_boot_count

            self._m_boot_count = (self.boot_count_raw & 8191)
            return getattr(self, '_m_boot_count', None)

        @property
        def boot_cause(self):
            if hasattr(self, '_m_boot_cause'):
                return self._m_boot_cause

            self._m_boot_cause = ((self.boot_count_raw & 57344) >> 13)
            return getattr(self, '_m_boot_cause', None)


    class Aausat4DataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if self._root.frame_length == 92:
                self.data_length = Aausat4.Aausat4LenT(self._io, self, self._root)

            self.csp_header = Aausat4.CspHeaderT(self._io, self, self._root)
            self.data = Aausat4.CspDataT(self._io, self, self._root)
            if  ((self._root.frame_length == 90) or (self._root.frame_length == 92)) :
                self.hmac = Aausat4.Aausat4HmacT(self._io, self, self._root)



    class Adcs1T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes(7)


    class Aausat4LenT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.data_plus_hmac_length = self._io.read_u2be()


    class CspDataT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if  ((self._parent.csp_header.dst_port == 10) and (self._parent.csp_header.destination == 9)) :
                self.beacon = Aausat4.Aausat4BeaconT(self._io, self, self._root)



    class Ais2T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes(20)


    class Aausat4BeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.valid = self._io.read_u1()
            self._raw_eps = self._io.read_bytes(20)
            _io__raw_eps = KaitaiStream(BytesIO(self._raw_eps))
            self.eps = Aausat4.EpsT(_io__raw_eps, self, self._root)
            self._raw_com = self._io.read_bytes(10)
            _io__raw_com = KaitaiStream(BytesIO(self._raw_com))
            self.com = Aausat4.ComT(_io__raw_com, self, self._root)
            self.adcs1 = Aausat4.Adcs1T(self._io, self, self._root)
            self.adcs2 = Aausat4.Adcs2T(self._io, self, self._root)
            self.ais1 = Aausat4.Ais1T(self._io, self, self._root)
            self.ais2 = Aausat4.Ais2T(self._io, self, self._root)


    class CspHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.csp_flags = []
            for i in range(4):
                self.csp_flags.append(self._io.read_u1())


        @property
        def source(self):
            if hasattr(self, '_m_source'):
                return self._m_source

            self._m_source = ((self.csp_flags[3] >> 1) & 31)
            return getattr(self, '_m_source', None)

        @property
        def rdp(self):
            if hasattr(self, '_m_rdp'):
                return self._m_rdp

            self._m_rdp = ((self.csp_flags[0] >> 1) & 1)
            return getattr(self, '_m_rdp', None)

        @property
        def src_port(self):
            if hasattr(self, '_m_src_port'):
                return self._m_src_port

            self._m_src_port = (self.csp_flags[1] & 63)
            return getattr(self, '_m_src_port', None)

        @property
        def destination(self):
            if hasattr(self, '_m_destination'):
                return self._m_destination

            self._m_destination = (((self.csp_flags[2] | (self.csp_flags[3] << 8)) >> 4) & 31)
            return getattr(self, '_m_destination', None)

        @property
        def dst_port(self):
            if hasattr(self, '_m_dst_port'):
                return self._m_dst_port

            self._m_dst_port = (((self.csp_flags[1] | (self.csp_flags[2] << 8)) >> 6) & 63)
            return getattr(self, '_m_dst_port', None)

        @property
        def priority(self):
            if hasattr(self, '_m_priority'):
                return self._m_priority

            self._m_priority = (self.csp_flags[3] >> 6)
            return getattr(self, '_m_priority', None)

        @property
        def reserved(self):
            if hasattr(self, '_m_reserved'):
                return self._m_reserved

            self._m_reserved = (self.csp_flags[0] >> 4)
            return getattr(self, '_m_reserved', None)

        @property
        def xtea(self):
            if hasattr(self, '_m_xtea'):
                return self._m_xtea

            self._m_xtea = ((self.csp_flags[0] >> 2) & 1)
            return getattr(self, '_m_xtea', None)

        @property
        def hmac(self):
            if hasattr(self, '_m_hmac'):
                return self._m_hmac

            self._m_hmac = ((self.csp_flags[0] >> 3) & 1)
            return getattr(self, '_m_hmac', None)

        @property
        def crc(self):
            if hasattr(self, '_m_crc'):
                return self._m_crc

            self._m_crc = (self.csp_flags[0] & 1)
            return getattr(self, '_m_crc', None)


    class Adcs2T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes(6)


    class Ais1T(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unparsed = self._io.read_bytes(20)


    @property
    def frame_length(self):
        if hasattr(self, '_m_frame_length'):
            return self._m_frame_length

        self._m_frame_length = self._io.size()
        return getattr(self, '_m_frame_length', None)


